__all__ = [
    "create_client",
    "Client",
    "AsyncClient",
    "exceptions",
]

import socket
from typing import List, Optional
from uuid import uuid4

from .client_async import AsyncClient
from .redis_rs import Client, exceptions
from .redis_rs import create_client as _create_client


def create_client(
    *args: str,
    host: str = "localhost",
    port: int = 6379,
    max_size: Optional[int] = None,
    cluster: Optional[bool] = None,
    username: Optional[str] = None,
    password: Optional[str] = None,
    db: Optional[int] = None,
    client_id: Optional[str] = None,
    features: Optional[List[str]] = None,
    max_delay_ms: Optional[int] = None,
) -> Client:
    if not client_id:
        client_id = f"{socket.gethostname()}-{uuid4()}"
    return _create_client(
        *args,
        host=host,
        port=port,
        max_size=max_size,
        cluster=cluster,
        username=username,
        password=password,
        db=db,
        client_id=client_id,
        features=features,
        max_delay=max_delay_ms,
    )



from typing import Literal, Union

Encoding = Union[
    Literal["utf-8"],
    Literal["utf8"],
    Literal["int"],
    Literal["float"],
    Literal["info"],
    Literal["json"],
]
Arg = Union[str, bytes, int, float]
Result = Union[bytes, str, int, float, dict, list]



__all__ = [
    "create_client",
    "Client",
    "AsyncClient",
    "exceptions",
]

from typing import Dict, List, Optional

from redis_rs.client_async import AsyncClient

class exceptions:
    class PoolError(Exception): ...
    class RedisError(Exception): ...

class Client:
    def status(self) -> Dict: ...
    async def __aenter__(self) -> AsyncClient: ...
    async def __aexit__(self, *args, **kwargs): ...

def create_client(
    *args: str,
    host: str = "localhost",
    port: int = 6379,
    max_size: Optional[int] = None,
    cluster: Optional[bool] = None,
    username: Optional[str] = None,
    password: Optional[str] = None,
    db: Optional[int] = None,
    client_id: Optional[str] = None,
    features: Optional[List[str]] = None,
    max_delay_ms: Optional[int] = None,
) -> Client: ...



from uuid import uuid4

import redis_rs


async def test_pfadd(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    result = await async_client.pfadd(key, 2)
    assert result is True


async def test_pfcount(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    b = await async_client.pfadd(key, 2)
    assert b is True

    result = await async_client.pfcount(key)
    assert result == 1


async def test_pfmerge(async_client: redis_rs.AsyncClient):
    key1 = str(uuid4())
    key2 = str(uuid4())

    b = await async_client.pfadd(key1, 2)
    assert b is True

    b = await async_client.pfmerge(key2, key1)
    assert b is True

    result = await async_client.pfcount(key2)
    assert result == 1



import asyncio
import os

import pytest

import redis_rs


def to_addr(s: str) -> str:
    if s.isdigit():
        return f"redis://localhost:{s}"
    elif not s.startswith("redis://"):
        return f"redis://{s}"
    return s


async def get_redis_version(nodes: list) -> str:
    key = "redis_version"
    async with redis_rs.create_client(
        *nodes,
        features=FEATURES,
    ) as c:
        infos = await c.execute("INFO", "SERVER", encoding="info")
        if isinstance(infos, dict):
            result = infos.get(key, "")
        else:
            assert isinstance(infos, list)
            info = filter(lambda x: isinstance(x, dict), infos)
            result = min(map(lambda x: x[key], info))
        return result


NODES = [to_addr(node) for node in os.environ.get("REDIS_NODES", "").split(",") if node]
IS_CLUSTER = os.environ.get("REDIS_CLUSTER", "0") not in {"0"}
FEATURES = [f for f in os.environ.get("REDIS_RS_FEATURES", "").split(",") if f]
VERSION = ""


@pytest.fixture
def client_factory():
    def factory(**kwargs):
        features = kwargs.setdefault("features", [])
        features.extend(FEATURES)
        return redis_rs.create_client(
            *NODES,
            cluster=IS_CLUSTER,
            **kwargs,
        )

    return factory


@pytest.fixture
async def async_client(client_factory):
    async with client_factory() as c:
        yield c


def pytest_runtest_setup(item):
    for marker in item.iter_markers():
        if marker.name == "redis":
            if marker.kwargs.get("cluster") and not IS_CLUSTER:
                pytest.skip("Single redis")
            elif marker.kwargs.get("single") and IS_CLUSTER:
                pytest.skip("Cluster redis")
            elif version := marker.kwargs.get("version"):
                global VERSION
                if not VERSION:
                    VERSION = asyncio.run(get_redis_version(NODES))
                if str(version) > VERSION:
                    pytest.skip(f"redis_version:{VERSION} < {version}")



from uuid import uuid4

import pytest

import redis_rs


async def test_hgetall(async_client: redis_rs.AsyncClient):
    key = uuid4().hex

    await async_client.hset(key, "a", 1)
    await async_client.hset(key, "b", 2)
    result = await async_client.fetch_dict("HGETALL", key)
    assert result
    assert result == {"a": b"1", "b": b"2"}


async def test_zrange(async_client: redis_rs.AsyncClient):
    key = uuid4().hex

    await async_client.execute("ZADD", key, 1.5678, "b")
    await async_client.execute("ZADD", key, 2.6, "a")
    result = await async_client.fetch_dict("ZRANGE", key, 0, -1, "WITHSCORES", encoding="float")
    assert result
    assert result == {"a": 2.6, "b": 1.5678}


async def test_xread(async_client: redis_rs.AsyncClient):
    stream = uuid4().hex

    result = await async_client.xread({stream: 0})
    assert isinstance(result, dict)
    assert result == {}

    ident = await async_client.xadd(stream, {"a": "bcd"})
    assert isinstance(ident, str)

    result = await async_client.xread(stream)
    assert isinstance(result, dict)
    assert result == {stream: {ident: {"a": b"bcd"}}}


async def test_xinfo_stream(async_client: redis_rs.AsyncClient):
    stream = f"stream-{uuid4()}"
    group = f"group-{uuid4()}"

    await async_client.execute("XGROUP", "CREATE", stream, group, 0, "MKSTREAM")
    xinfo = await async_client.fetch_dict("XINFO", "STREAM", stream)
    assert xinfo
    assert len(xinfo) > 6


async def test_xinfo_groups(async_client: redis_rs.AsyncClient):
    stream = f"stream-{uuid4()}"
    group = f"group-{uuid4()}"

    await async_client.execute("XGROUP", "CREATE", stream, group, 0, "MKSTREAM")
    xinfo = await async_client.fetch_dict("XINFO", "GROUPS", stream)
    assert xinfo
    assert len(xinfo) == 1
    assert len(xinfo[0]) > 3

    group = f"group-{uuid4()}"
    await async_client.execute("XGROUP", "CREATE", stream, group, 0, "MKSTREAM")
    xinfo = await async_client.fetch_dict("XINFO", "GROUPS", stream)
    assert xinfo
    assert len(xinfo) == 2
    assert len(xinfo[0]) > 3
    assert len(xinfo[1]) > 3


@pytest.mark.redis(version=6)
async def test_xinfo_consumers(async_client: redis_rs.AsyncClient):
    stream = f"stream-{uuid4()}"
    group = f"group-{uuid4()}"
    consumer = f"consumer-{uuid4()}"

    await async_client.execute("XGROUP", "CREATE", stream, group, 0, "MKSTREAM")
    await async_client.execute("XGROUP", "CREATECONSUMER", stream, group, consumer)
    xinfo = await async_client.fetch_dict("XINFO", "CONSUMERS", stream, group)
    assert xinfo
    assert len(xinfo) == 1
    assert len(xinfo[0]) > 2


@pytest.mark.redis(version=7, cluster=True)
async def test_cluster_shards(async_client: redis_rs.AsyncClient):
    result = await async_client.fetch_dict("CLUSTER", "SHARDS")
    assert result
    assert len(result) > 1
    assert len(result[0]) > 1


@pytest.mark.redis(cluster=True)
async def test_info(async_client: redis_rs.AsyncClient):
    result = await async_client.fetch_dict("INFO", "SERVER", encoding="info")
    assert result
    assert isinstance(result, dict)
    for k, v in result.items():
        assert isinstance(k, str)
        assert "redis_version" in v



from uuid import uuid4

import pytest

import redis_rs


@pytest.mark.redis(single=True)
async def test_parse_info(async_client: redis_rs.AsyncClient):
    result = await async_client.execute("INFO", encoding="info")
    assert isinstance(result, dict)
    assert "redis_version" in result


async def test_parse_json(async_client: redis_rs.AsyncClient):
    key = uuid4().hex
    await async_client.set(key, '{"a": 3, "i": ["b"]}')
    result = await async_client.get(key, encoding="json")
    assert isinstance(result, dict)
    assert result == {"a": 3, "i": ["b"]}



from uuid import uuid4

import redis_rs


async def test_set(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    await async_client.set(key, 1)
    result = await async_client.get(key)
    assert result == b"1"


async def test_get_int(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    await async_client.set(key, 3)
    result = await async_client.get(key, encoding="int")
    assert result == 3


async def test_set_ex(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    ttl = await async_client.fetch_int("TTL", key)
    assert ttl < 0
    await async_client.set(key, 2, ex=10)
    result = await async_client.get(key)
    assert result == b"2"
    ttl = await async_client.fetch_int("TTL", key)
    assert ttl > 0


async def test_set_px(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    ttl = await async_client.fetch_int("TTL", key)
    assert ttl < 0
    await async_client.set(key, 2, px=10000)
    result = await async_client.get(key)
    assert result == b"2"
    ttl = await async_client.fetch_int("TTL", key)
    assert ttl > 0



from uuid import uuid4

import pytest

import redis_rs


async def test_lpush(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    n = await async_client.lpush(key, 2)
    assert n == 1

    n = await async_client.lpush(key, 3)
    assert n == 2

    result = await async_client.lrange(key, encoding="int")
    assert result == [3, 2]


async def test_rpush(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    n = await async_client.rpush(key, 2)
    assert n == 1

    n = await async_client.rpush(key, 3)
    assert n == 2

    result = await async_client.lrange(key, encoding="int")
    assert result == [2, 3]


async def test_lrange(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    n = await async_client.lpush(key, 2)
    assert n == 1

    n = await async_client.lpush(key, 1)
    assert n == 2

    n = await async_client.rpush(key, 3)
    assert n == 3

    result = await async_client.lrange(key, 1, 1, encoding="int")
    assert result == [2]

    result = await async_client.lrange(key, 0, -1, encoding="int")
    assert result == [1, 2, 3]

    result = await async_client.lrange(key, encoding="int")
    assert result == [1, 2, 3]


@pytest.mark.redis(version=6.2)
async def test_lpop(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    n = await async_client.lpush(key, 2)
    assert n == 1

    n = await async_client.lpush(key, 3)
    assert n == 2

    result = await async_client.lpop(key, encoding="int")
    assert result == 3

    n = await async_client.lpush(key, 4)
    assert n == 2

    result = await async_client.lpop(key, count=3, encoding="int")
    assert result == [4, 2]

    result = await async_client.lpush(key, 5)
    assert result == 1


@pytest.mark.redis(version=6)
async def test_blpop(async_client: redis_rs.AsyncClient):
    key1 = str(uuid4()) + "{a}"
    key2 = str(uuid4()) + "{a}"

    n = await async_client.lpush(key1, 1)
    assert n == 1

    n = await async_client.lpush(key2, 2)
    assert n == 1

    result = await async_client.blpop(key1, key2, timeout=0, encoding="int")
    assert result == [None, 1]

    result = await async_client.blpop(key1, key2, timeout=0, encoding="int")
    assert result == [None, 2]


async def test_llen(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    result = await async_client.lpush(key, 2)
    assert result == 1

    result = await async_client.rpush(key, 3)
    assert result == 2

    result = await async_client.llen(key)
    assert result == 2


async def test_lrem(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    n = await async_client.lpush(key, 2)
    assert n == 1

    n = await async_client.lpush(key, 3)
    assert n == 2

    result = await async_client.lrem(key, 0, 2)
    assert result == 1



from typing import Dict, List, Literal, Mapping, Optional, Union, overload

from redis_rs.types import Arg, Encoding, Result

class AsyncClient:
    client_id: str
    def status(self) -> Dict: ...
    async def execute(self, *args: Arg, encoding: Optional[Encoding] = None) -> Result: ...
    async def fetch_bytes(self, *args: Arg) -> bytes: ...
    async def fetch_str(self, *args: Arg) -> str: ...
    async def fetch_int(self, *args: Arg) -> int: ...
    async def fetch_float(self, *args: Arg) -> float: ...
    async def fetch_dict(self, *args: Arg, encoding: Optional[Encoding] = None) -> dict: ...
    async def fetch_scores(self, *args: Arg) -> Dict[str, float]: ...
    async def exists(self, *keys: str) -> int: ...
    async def expire(self, key: str, seconds: int, option: Optional[str] = None) -> int: ...
    async def delete(self, *keys: str) -> int: ...
    async def keys(self, pattern: str, encoding: Optional[str] = None) -> List[str]: ...
    async def eval(
        self, script: str, numkeys: int, *keys_and_args: Arg, encoding: Optional[Encoding] = None
    ) -> Result: ...
    @overload
    async def set(
        self,
        key: str,
        value: Arg,
        ex: Optional[int] = None,
        encoding: Optional[Encoding] = None,
    ) -> Result: ...
    @overload
    async def set(
        self,
        key: str,
        value: Arg,
        px: Optional[int] = None,
        encoding: Optional[Encoding] = None,
    ) -> Result: ...
    async def get(self, key: str, *, encoding: Optional[Encoding] = None) -> Result: ...
    @overload
    async def hset(self, key: str, field: str, value: Arg, *pairs) -> int: ...
    @overload
    async def hset(self, key: str, mapping: Dict[str, Arg]) -> int: ...
    async def hget(self, key: str, field: str, *, encoding: Optional[Encoding] = None) -> Result: ...
    async def hmget(self, key: str, *fields: str, encoding: Optional[Encoding] = None) -> Result: ...
    async def hgetall(self, key: str, *, encoding: Optional[Encoding] = None) -> Dict: ...
    async def hdel(self, key: str, *fields: str) -> int: ...
    async def hexists(self, key: str, field: str) -> bool: ...
    async def incr(self, key: str, delta: Union[None, int, float] = None) -> float: ...
    async def lpush(self, key: str, value: Arg) -> int: ...
    async def rpush(self, key: str, value: Arg) -> int: ...
    async def lpop(self, key: str, *, count=None, encoding: Optional[Encoding] = None) -> Result: ...
    async def lrem(self, key: str, count: int, element: Arg) -> Result: ...
    async def blpop(self, *keys: str, timeout, encoding: Optional[Encoding] = None) -> Result: ...
    async def lrange(
        self,
        key: str,
        start: int = 0,
        stop: int = -1,
        *,
        encoding: Optional[Encoding] = None,
    ) -> List[Result]: ...
    async def llen(self, key: str) -> int: ...
    async def pfadd(self, key: str, *elements: Arg) -> bool: ...
    async def pfcount(self, *keys: str) -> int: ...
    async def pfmerge(self, destkey: str, *sourcekeys: str) -> bool: ...
    @overload
    async def xadd(
        self,
        stream: str,
        id: str,
        items: Dict[str, Arg],
        *,
        mkstream: bool = True,
        maxlen: Optional[int] = None,
        minid: Optional[int] = None,
        approx: bool = True,
        limit: Optional[int] = None,
    ) -> str: ...
    @overload
    async def xadd(
        self,
        stream: str,
        items: Dict[str, Arg],
        *,
        id: str = "*",
        mkstream: bool = True,
        maxlen: Optional[int] = None,
        minid: Optional[int] = None,
        approx: bool = True,
        limit: Optional[int] = None,
    ) -> str: ...
    @overload
    async def xadd(
        self,
        stream: str,
        *args: Arg,
        id: str = "*",
        mkstream: bool = True,
        maxlen: Optional[int] = None,
        minid: Optional[int] = None,
        approx: bool = True,
        limit: Optional[int] = None,
    ) -> str: ...
    @overload
    async def xread(
        self,
        streams: Dict[str, Union[str, Literal["$"], Literal[">"], Literal[0]]],
        *,
        block: Optional[int] = None,
        count: Optional[int] = None,
        noack: Optional[bool] = None,
        group: Optional[str] = None,
        encoding: Optional[Encoding] = None,
    ) -> Dict: ...
    @overload
    async def xread(
        self,
        *streams: str,
        id: Union[None, str, Literal["$"], Literal[">"], Literal[0]] = None,
        block: Optional[int] = None,
        count: Optional[int] = None,
        noack: Optional[bool] = None,
        group: Optional[str] = None,
        encoding: Optional[Encoding] = None,
    ) -> Dict: ...
    async def xack(
        self,
        key: str,
        group: str,
        *id: Union[str, Literal["$"], Literal[0]],
    ) -> int: ...
    @overload
    async def zadd(
        self,
        key: str,
        score: float,
        value: str,
        *pairs: Arg,
    ) -> int: ...
    @overload
    async def zadd(
        self,
        key: str,
        *args: Arg,
        score: Optional[float] = None,
        incr: Optional[float] = None,
    ) -> int: ...
    @overload
    async def zadd(
        self,
        key: str,
        *args: Union[Mapping[str, Arg], Mapping[bytes, Arg]],
        incr: Optional[float] = None,
    ) -> int: ...
    @overload
    async def zrange(
        self,
        key: str,
        start: Union[int, str] = 0,
        stop: Union[int, str] = -1,
    ) -> List[Result]: ...
    @overload
    async def zrange(
        self,
        key: str,
        start: Union[int, str] = 0,
        stop: Union[int, str] = -1,
        *,
        withscores: Literal[True],
    ) -> Dict[str, float]: ...
    async def zcard(self, key: str) -> int: ...
    async def zrem(self, key: str, *members: str) -> int: ...



from uuid import uuid4

import redis_rs


async def test_exists(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    await async_client.set(key, 1)

    result = await async_client.exists(key)
    assert result == 1


async def test_delete(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    result = await async_client.delete(key)
    assert result == 0

    await async_client.set(key, 1)

    result = await async_client.delete(key)
    assert result == 1


async def test_keys(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    await async_client.set(key, 1)

    result = await async_client.keys(key)
    assert result == [key]


async def test_expire(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    await async_client.set(key, 1)

    result = await async_client.expire(key, 2)
    assert result is True



import time
from uuid import uuid4

import pytest

import redis_rs


@pytest.mark.redis(version=6.2)
async def test_xadd(async_client: redis_rs.AsyncClient):
    stream = str(uuid4())

    ident = await async_client.xadd(stream, {"a": "bcd"})
    assert isinstance(ident, str)

    ident = await async_client.xadd(stream, {"a": "bcd"}, maxlen=2, approx=True, limit=5)
    assert isinstance(ident, str)

    ident = await async_client.xadd(stream, {"a": "bcd"}, maxlen=2, approx=True)
    assert isinstance(ident, str)

    ident = await async_client.xadd(stream, {"a": "bcd"}, minid=2, approx=True, limit=5)
    assert isinstance(ident, str)

    ident = await async_client.xadd(stream, {"a": "bcd"}, minid=2, approx=True)
    assert isinstance(ident, str)


@pytest.mark.redis(version=6.2)
async def test_xadd_nomkstream(async_client: redis_rs.AsyncClient):
    stream = str(uuid4())

    ident = await async_client.xadd(stream, {"a": "bcd"}, mkstream=False)
    assert ident is None


@pytest.mark.redis(version=6.2)
async def test_xadd_flat(async_client: redis_rs.AsyncClient):
    stream = str(uuid4())

    ident = await async_client.xadd(stream, "a", "b", "c", "d")
    assert isinstance(ident, str)


@pytest.mark.redis(version=7)
@pytest.mark.parametrize("id", ["2-*", b"3-*"])
async def test_xadd_flat_id_star(id, async_client: redis_rs.AsyncClient):
    stream = str(uuid4())

    ident = await async_client.xadd(stream, id, "a", "b", minid=1)
    assert isinstance(ident, str), ident


@pytest.mark.redis(version=6.2)
@pytest.mark.parametrize(
    "id",
    [
        1,
        1.0,
        1.01,
        "1-0",
        "1-1",
        1.00001,
        time.time(),
        int(time.time()),
        time.time() * 1000,
        str(int(time.time())),
        str(time.time()).replace(".", "-"),
        str(int(time.time())).encode("utf-8"),
    ],
)
async def test_xadd_flat_id(id, async_client: redis_rs.AsyncClient):
    stream = str(uuid4())

    ident = await async_client.xadd(stream, id, "a", "b", minid=1)
    assert isinstance(ident, str), ident


async def test_xread(async_client: redis_rs.AsyncClient):
    stream = str(uuid4()) + "{a}"
    stream1 = str(uuid4()) + "{a}"
    stream2 = str(uuid4()) + "{a}"

    result = await async_client.xread({stream: 0})
    assert result == {}

    ident = await async_client.xadd(stream, {"a": "bcd"})
    assert isinstance(ident, str)

    result = await async_client.xread({stream: 0})
    assert result == {stream: {ident: {"a": b"bcd"}}}
    assert isinstance(result, dict)

    result = await async_client.xread({stream: 0}, encoding="utf-8")
    assert result == {stream: {ident: {"a": "bcd"}}}
    assert isinstance(result, dict)

    result = await async_client.xread(stream, encoding="utf-8")
    assert result == {stream: {ident: {"a": "bcd"}}}
    assert isinstance(result, dict)

    result = await async_client.xread(stream, id=0, encoding="utf-8")
    assert result == {stream: {ident: {"a": "bcd"}}}
    assert isinstance(result, dict)

    result = await async_client.xread(stream, id="$", encoding="utf-8")
    assert result == {}
    assert isinstance(result, dict)

    result = await async_client.xread(stream1, stream, stream2, encoding="utf-8")
    assert result == {stream: {ident: {"a": "bcd"}}}
    assert isinstance(result, dict)

    result = await async_client.xread(stream1, stream, stream2, id="0", encoding="utf-8")
    assert result == {stream: {ident: {"a": "bcd"}}}
    assert isinstance(result, dict)

    result = await async_client.xread(stream1, stream, stream2, id="$", encoding="utf-8")
    assert result == {}
    assert isinstance(result, dict)


async def test_xreadgroup(async_client: redis_rs.AsyncClient):
    stream = f"stream-{uuid4()}"
    group = f"group-{uuid4()}"

    result = await async_client.xread(stream)
    assert result == {}

    ident = await async_client.xadd(stream, {"a": "1"})
    assert isinstance(ident, str)

    result = await async_client.xread(stream)
    assert result
    assert result == {stream: {ident: {"a": b"1"}}}
    assert isinstance(result, dict)

    await async_client.execute("XGROUP", "CREATE", stream, group, "$", "MKSTREAM")

    ident = await async_client.xadd(stream, {"a": "2"})
    assert isinstance(ident, str)

    result = await async_client.xread(stream, group=group)
    assert result == {stream: {ident: {"a": b"2"}}}
    assert await async_client.xack(stream, group, ident)
    assert isinstance(result, dict)

    ident = await async_client.xadd(stream, {"a": "3"})
    assert isinstance(ident, str)

    result = await async_client.xread(stream, group=group, count=2, block=5000)
    assert result == {stream: {ident: {"a": b"3"}}}
    assert isinstance(result, dict)



import pytest

import redis_rs


async def test_client_id(async_client: redis_rs.AsyncClient):
    assert async_client.client_id


@pytest.mark.redis(single=True, version=6)
async def test_password(async_client: redis_rs.AsyncClient, client_factory):
    user = "test"
    password = await async_client.fetch_str("ACL", "GENPASS")
    assert await async_client.execute("ACL", "SETUSER", user, "nopass")
    assert await async_client.execute("ACL", "SETUSER", user, "on", ">" + password, "+acl|whoami", "+cluster|slots")
    assert await async_client.execute("AUTH", user, password)
    assert user == await async_client.fetch_str("ACL", "WHOAMI")

    async with client_factory(username=user, password=password) as client:
        assert user == await client.fetch_str("ACL", "WHOAMI")
        status = client.status()
        assert status.get("username")
        assert status.get("auth")



import pytest

from redis_rs import AsyncClient
from redis_rs.exceptions import RedisError


async def test_redis_error(async_client: AsyncClient):
    with pytest.raises(RedisError):
        await async_client.fetch_int("CLUSTER", "SLOTS")



from uuid import uuid4

import redis_rs


async def test_hgetall(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    result = await async_client.hgetall(key)
    assert result == {}
    assert isinstance(result, dict)

    await async_client.hset(key, "f", 123.456)
    result = await async_client.hgetall(key)
    assert result == {"f": b"123.456"}
    assert isinstance(result, dict)

    result = await async_client.hgetall(key, encoding="float")
    assert result == {"f": 123.456}
    assert isinstance(result, dict)


async def test_hset_map(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    n = await async_client.hset(key, {"a": "2", "b": b"3"})
    assert n == 2
    n = await async_client.hset(key, mapping={"x": "1", "z": b"4"})
    assert n == 2
    result = await async_client.hgetall(key, encoding="float")
    assert result == {"a": 2.0, "b": 3.0, "x": 1.0, "z": 4.0}
    assert isinstance(result, dict)


async def test_hset_pairs(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    n = await async_client.hset(key, "x", "2", "y", b"3")
    assert n == 2
    result = await async_client.hgetall(key, encoding="int")
    assert result == {"x": 2, "y": 3}
    assert isinstance(result, dict)


async def test_hdel(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    n = await async_client.hset(key, "x", "2", "y", b"3")
    assert n == 2
    n = await async_client.hdel(key, "x")
    assert n == 1
    n = await async_client.hdel(key, "x")
    assert n == 0
    result = await async_client.hgetall(key, encoding="int")
    assert result == {"y": 3}
    assert isinstance(result, dict)


async def test_hexists(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    result = await async_client.hset(key, "x", "2", "y", b"3")
    assert result == 2
    result = await async_client.hexists(key, "x")
    assert result is True
    result = await async_client.hexists(key, "z")
    assert result is False


async def test_hmget(async_client: redis_rs.AsyncClient):
    key = str(uuid4())
    n = await async_client.hset(key, "x", "2", "y", b"3", "z", 4)
    assert n == 3
    result = await async_client.hmget(key, "x", "z", encoding="int")
    assert result == [2, 4]



from uuid import uuid4

import redis_rs


async def test_zadd(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    n = await async_client.zadd(key, 2, "a")
    assert n == 1

    n = await async_client.zadd(key, "b", score=3)
    assert n == 1

    n = await async_client.zadd(key, {"c": 4})
    assert n == 1

    n = await async_client.zadd(key, {b"d": 4})
    assert n == 1

    result = await async_client.execute("ZRANGE", key, 0, -1, encoding="utf8")
    assert result == ["a", "b", "c", "d"]


async def test_zrange(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    n = await async_client.zadd(key, "a", score=1)
    assert n == 1

    n = await async_client.zadd(key, "b", score=2)
    assert n == 1

    result = await async_client.zrange(key)
    assert result == [b"a", b"b"]

    result_d = await async_client.zrange(key, withscores=True)
    assert result_d == {"a": 1, "b": 2}


async def test_zcard(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    n = await async_client.zadd(key, "a", score=1)
    assert n == 1

    n = await async_client.zadd(key, "b", score=2)
    assert n == 1

    result = await async_client.zcard(key)
    assert result == 2


async def test_zrem(async_client: redis_rs.AsyncClient):
    key = str(uuid4())

    n = await async_client.zadd(key, "a", score=1)
    assert n == 1

    n = await async_client.zadd(key, "b", score=2)
    assert n == 1

    result = await async_client.zcard(key)
    assert result == 2

    result = await async_client.zrem(key, "a")
    assert result == 1

    result = await async_client.zrem(key, "z")
    assert result == 0

    result = await async_client.zcard(key)
    assert result == 1



from uuid import uuid4

import pytest

import redis_rs


async def test_execute(async_client: redis_rs.AsyncClient):
    key = uuid4().hex
    result = await async_client.execute("SET", key, 1)
    result = await async_client.execute("GET", key)
    assert isinstance(result, bytes)

    result = await async_client.execute("GET", key, encoding="utf-8")
    assert isinstance(result, str)


@pytest.mark.redis(single=True)
async def test_fetch_str_info(async_client: redis_rs.AsyncClient):
    result = await async_client.fetch_str("INFO")
    assert isinstance(result, str)
    assert "redis_version" in result
